#-*- coding: utf-8 -*-
# This file is automatically generated by 'setup.py'
"""
Created on 2017-01-05 20:53:15
Modified on 2018-11-08 18:27:37

@author: Young Ju Kim
"""


__all__ = ['__version__']


__version__ = '0.1.12'

